import argparse
from abc import ABC, abstractmethod
from dataclasses import dataclass

from polyglot import license

ACTIONS: list[str] = [
    "translate",
    "set_license",
    "print_supported_languages",
    "print_usage_info",
]


@dataclass
class Arguments:
    action: str
    source_file: str
    target_lang: str
    output_directory: str
    source_lang: str
    license_manager: license.LicenseManager


class ArgumentsCollector(ABC):
    arguments: Arguments

    def __init__(self) -> None:
        self._collect_arguments()
        self._validate_arguments()

    @abstractmethod
    def _collect_arguments() -> None:
        pass

    @abstractmethod
    def _validate_arguments() -> None:
        pass


class CLIArgumentsCollector(ArgumentsCollector):

    __parser: argparse.ArgumentParser
    __namespace: argparse.Namespace

    def _collect_arguments(self) -> None:
        self.__set_parser()
        self.__namespace = self.__parser.parse_args()
        self.arguments = Arguments(
            action=self.__namespace.action,
            source_file=self.__namespace.source_file,
            target_lang=self.__namespace.target_lang,
            output_directory=self.__namespace.output_directory,
            source_lang=self.__namespace.source_lang,
            license_manager=license.CLILicenseManager(),
        )

    def _validate_arguments(self) -> None:
        if self.__namespace.action == "translate" and (
            not self.__namespace.source_file or not self.__namespace.target_lang
        ):
            self.__parser.error("translate requires --source_file and --target_lang.")

    def __set_parser(self) -> None:

        parser: argparse.ArgumentParser = argparse.ArgumentParser(
            description="Polyglot will translate the given files."
        )

        parser.add_argument(
            "action",
            type=str,
            help="The command that will be exectued. The following options are for the translate command.",
            choices=ACTIONS,
        )

        parser.add_argument(
            "-p",
            "--source_file",
            type=str,
            help='The file to be translated. Required if the action is "translate."',
        )

        parser.add_argument(
            "-t",
            "--target_lang",
            type=str,
            help='The code of the language into which you want to translate the source file. Required if the action is "translate".',
        )

        parser.add_argument(
            "-o",
            "--output_directory",
            type=str,
            help="The directory where the output file will be located. Will be used the working directory if this option is invalid or not used.",
        )

        parser.add_argument(
            "-s",
            "--source_lang",
            type=str,
            help="Source file language code. Detected automatically by DeepL by default. Specifying it can increase performance and make translations more accurate.",
        )
        self.__parser = parser



from email import message
import requests
import json
import colorama
from requests.models import Response

import polyglot
from polyglot import license
from polyglot.errors import DeeplError
from polyglot.utilities import get_color_by_percentage, get_truncated_text


class Deepl:

    __LEN_LIMIT: int = 150

    __license: license.License
    __license_manager: license.LicenseManager

    def __init__(
        self,
        license_manager: license.LicenseManager,
    ) -> None:
        self.__license_manager = license_manager
        self.__license = self.__license_manager.get_license()

    @property
    def __base_url(self) -> str:
        version: str = "" if self.__license.version == "pro" else "-free"
        return f"https://api{version}.deepl.com/v2/"

    @property
    def __headers(self) -> dict[str, str]:
        return {
            "Authorization": f"DeepL-Auth-Key {self.__license.key}",
            "Content-Type": "application/json",
        }

    def __get_usage_info(self) -> Response:
        return requests.get(f"{self.__base_url}usage", headers=self.__headers)

    def print_usage_info(self) -> None:
        response: Response = self.__get_usage_info()

        try:
            body: dict[str, int] = json.loads(response.text)
            character_count: int = body["character_count"]
            character_limit: int = body["character_limit"]
            percentage: int = round((character_count / character_limit) * 100)
            print_color: str = get_color_by_percentage(percentage)
            print(
                f"\nPolyglot version: {polyglot.__version__}\nAPI key: {self.__license.key}\nCharacters limit: {character_limit}\n{print_color}Used characters: {character_count} ({percentage}%)\n"
            )

        except:
            raise DeeplError(
                status_code=response.status_code, message="Error retrieving usage info"
            )

    def print_supported_languages(self) -> None:
        response: Response = requests.get(
            f"{self.__base_url}languages", headers=self.__headers
        )

        try:
            body: dict = json.loads(response.text)

            for lang in body:
                print(f"{lang['name']} ({lang['language']})")

        except:
            raise DeeplError(
                status_code=response.status_code,
                message="Error retrieving the supported languages.",
            )

    def translate(self, entry: str, target_lang: str, source_lang: str = "") -> str:

        endpoint: str = f"{self.__base_url}translate?auth_key={self.__license.key}&text={entry}&target_lang={target_lang}"

        if source_lang != "":
            endpoint += f"&source_lang={source_lang}"

        response: Response = requests.get(endpoint)
        truncated_text: str = get_truncated_text(entry, self.__LEN_LIMIT)

        try:

            body: dict = json.loads(response.text)
            translation: str = body["translations"][0]["text"]

            truncated_translation: str = get_truncated_text(
                translation, self.__LEN_LIMIT
            )
            print(f'"{truncated_text}" => "{truncated_translation}"')
            return translation

        except KeyError:
            message: str = json.loads(response.text)["message"]
            if message:
                raise DeeplError(
                    status_code=response.status_code,
                    message=f'Error translating "{truncated_text}". Message: {message}"\n',
                )
            print(
                f'{colorama.Fore.YELLOW}\nNo traslation found for "{truncated_text}"!\n'
            )

        except:
            raise DeeplError(
                status_code=response.status_code,
                message=f'Error translating "{truncated_text}".\n',
            )
        return ""

    def translate_document(
        self, source_file: str, target_lang: str, source_lang: str = ""
    ) -> dict[str, str]:
        request_data: dict[str, str] = {
            "target_lang": target_lang,
            "auth_key": self.__license.key,
            "filename": source_file,
        }

        if source_lang != "":
            request_data["source_lang"] = source_lang

        endpoint: str = f"{self.__base_url}document/"

        with open(source_file, "rb") as document:
            response: Response = requests.post(
                endpoint, data=request_data, files={"file": document}
            )

            if response.status_code == 200:
                return json.loads(response.text)

        raise DeeplError(
            status_code=response.status_code,
            message=f'Error translating document "{source_file}"',
        )

    def check_document_status(
        self, document_id: str, document_key: str
    ) -> dict[str, str]:
        endpoint: str = f"{self.__base_url}document/{document_id}?auth_key={self.__license.key}&document_key={document_key}"
        response: Response = requests.post(endpoint)

        if response.status_code == 200:
            return json.loads(response.text)

        raise DeeplError(
            status_code=response.status_code,
            message="Error checking the status of a document",
        )

    def download_translated_document(
        self, document_id: str, document_key: str
    ) -> bytes:
        endpoint: str = f"{self.__base_url}document/{document_id}/result?auth_key={self.__license.key}&document_key={document_key}"
        response: Response = requests.post(endpoint)

        if response.status_code == 200:
            return response.content

        raise DeeplError(
            status_code=response.status_code, message="Error downlaoding a document."
        )



#!/usr/bin/env python3

from polyglot import arguments, polyglot


def main() -> None:
    try:
        collector: arguments.ArgumentsCollector = arguments.CLIArgumentsCollector()
        options: arguments.Arguments = collector.arguments
        polyglot.Polyglot(options).execute_command()
    except KeyboardInterrupt:
        print("\n\nInterrupted by user.")


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
import pathlib
from setuptools import find_packages, setup
import polyglot

HERE = pathlib.Path(__file__).parent
README = (HERE / "README.md").read_text()

setup(
    name="polyglot-translator",
    version=polyglot.__version__,
    description="Automation CLI tool that, using the DeepL API, generates a JSON or a PO file from a given source file.",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/riccardoFasan/polyglot",
    project_urls={
        "BugTracker": "https://github.com/riccardoFasan/polyglot/issues",
        "Homepage": "https://github.com/riccardoFasan/polyglot",
    },
    author="Riccardo Fasan",
    author_email="fasanriccardo21@gmail.com",
    license="MIT",
    classifiers=[
        "Programming Language :: Python :: 3.10",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
    packages=find_packages(exclude=("tests",)),
    include_package_data=True,
    install_requires=[
        "requests",
        "colorama",
        "polib",
        "progressbar2",
    ],
    entry_points={
        "console_scripts": [
            "polyglot=polyglot.__main__:main",
        ]
    },
)



import json
import pathlib
from enum import Enum
from abc import ABC, abstractmethod
from dataclasses import dataclass
from distutils.util import strtobool


class LicenseVersion(Enum):
    FREE = "free"
    PRO = "pro"


@dataclass
class License:
    key: str
    version: LicenseVersion = LicenseVersion.FREE


class LicenseManager(ABC):
    @abstractmethod
    def get_license(self) -> License:
        return License(key="", version=LicenseVersion.FREE)

    @abstractmethod
    def set_license(self) -> None:
        pass


class CLILicenseManager(LicenseManager):
    @property
    def __license_path(self) -> str:
        return f"{pathlib.Path.home()}/.deepl_api_key.json"

    def get_license(self) -> License:
        try:
            with open(self.__license_path, "r") as license_file:
                file_content: dict[str, str] = json.load(license_file)

                if file_content["key"] and file_content["version"]:
                    return License(
                        key=file_content["key"],
                        version=LicenseVersion(file_content["version"]),
                    )
                return self.__set_and_get_license()
        except:
            return self.__set_and_get_license()

    def __set_and_get_license(self) -> License:
        self.set_license()
        return self.get_license()

    def set_license(self) -> None:
        with open(self.__license_path, "w+") as license_file:
            key: str = input("Type here your Deepl API key: ")
            version: LicenseVersion = (
                LicenseVersion.PRO
                if self.__yes_no_input("Are you using the pro license?")
                else LicenseVersion.FREE
            )
            license: dict[str, str] = {"key": key.strip(), "version": version.value}
            license_file.write(json.dumps(license, indent=2))

    def __yes_no_input(self, question: str) -> bool:
        while True:
            user_input = input(question + " [y/n]: ")
            try:
                return bool(strtobool(user_input))
            except ValueError:
                print("Please use y/n or yes/no.\n")



import json
import os
import polib
from abc import ABC, abstractmethod
from typing import Any

from polyglot.errors import HandlerError

# TODO: Check if the file is empty
def verfiy_source(function: Any) -> Any:
    def function_wrapper(instance: FileHandler):
        try:
            return function(instance)
        except FileNotFoundError:
            raise HandlerError(instance.source_file, "File not found.")
        except:
            raise HandlerError(instance.source_file, "File not supported.")

    return function_wrapper


class FileHandler(ABC):

    source_file: str
    _target_file: str

    def __init__(
        self, source_file: str, output_directory: str, target_lang: str
    ) -> None:
        self.source_file = source_file
        self.target_lang = target_lang
        self.__set_target_file(output_directory, target_lang)

    @property
    def _extension(self) -> str:
        return os.path.splitext(self.source_file)[1]

    def __set_target_file(self, output_directory: str, target_lang: str) -> None:
        output_directory = (
            output_directory
            if output_directory and os.path.isdir(output_directory)
            else os.getcwd()
        )
        self._target_file = f"{output_directory}/{target_lang.lower()}{self._extension}"

    @abstractmethod
    def read(self) -> Any:
        pass

    @abstractmethod
    def write(self, translated_content: Any) -> None:
        pass


class TextHandler(FileHandler):
    @verfiy_source
    def read(self) -> str:
        with open(self.source_file, "r") as source:
            return source.read()

    def write(self, translated_content: str) -> None:
        with open(self._target_file, "w+") as destination:
            destination.write(translated_content)
            print(f"Generated {self._target_file}.")


class JSONHandler(FileHandler):
    @verfiy_source
    def read(self) -> dict:
        with open(self.source_file, "r") as source:
            return json.load(source)

    def write(self, translated_content: dict) -> None:
        with open(self._target_file, "w+") as destination:
            destination.write(json.dumps(translated_content, indent=2))
            print(f"Generated {self._target_file}.")


class POHandler(FileHandler):

    __content: dict[str, Any] = {}

    @verfiy_source
    def read(self) -> dict:

        pofile: polib.POFile = self.__pofile_source
        translatables: dict[str, str] = {}

        for entry in pofile:
            message: str = entry.msgid if entry.msgstr == "" else entry.msgstr
            self.__content[entry.msgid] = {
                "msgstr": message,
                "occurrences": entry.occurrences,
            }
            translatables[entry.msgid] = message

        return translatables

    @property
    def __pofile_source(self) -> polib.POFile:
        return polib.pofile(self.source_file)

    def write(self, translated_content: dict[str, str]) -> None:
        pofile: polib.POFile = polib.POFile()
        pofile.metadata = self.__pofile_source.metadata

        self.__update_content(translated_content)

        for key, value in self.__content.items():
            entry: polib.POEntry = polib.POEntry(
                msgid=key, msgstr=value["msgstr"], occurrences=value["occurrences"]
            )
            pofile.append(entry)

        pofile.save(self._target_file)
        mofile_path: str = f"{ os.path.splitext(self._target_file)[0]}.mo"
        pofile.save_as_mofile(mofile_path)

        print(f"Generated {self._target_file} and {mofile_path}.")

    def __update_content(self, translated_content: dict[str, str]) -> None:
        for key, value in translated_content.items():
            self.__content[key]["msgstr"] = value


class DocumentHandler(FileHandler):
    @verfiy_source
    def read(self) -> str:
        with open(self.source_file, "r") as source:
            return self.source_file

    def write(self, translated_content: bytes) -> None:
        with open(self._target_file, "wb+") as destination:
            destination.write(translated_content)
            print(f"Generated {self._target_file}.")
